@@ -335,6 +335,144 @@ tp_f_get_attributes (model_evidence_o *self, PyObject *)
   return ret;
 }
 
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief <i>has_tag</i> method implementation
+//! \param self Object
+//! \param args Argument list
+//! \return true/false
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+static PyObject *
+tp_f_has_tag (model_evidence_o *self, PyObject *args)
+{
+  // Parse input args
+  std::string arg_name;
+
+  try
+    {
+      arg_name = mobius::py::get_arg_as_std_string (args, 0);
+    }
+  catch (const std::exception& e)
+    {
+      mobius::py::set_invalid_type_error (e.what ());
+      return nullptr;
+    }
+
+  // Execute C++ function
+  PyObject *ret = nullptr;
+
+  try
+    {
+      ret = mobius::py::pybool_from_bool (self->obj->has_tag (arg_name));
+    }
+  catch (const std::exception& e)
+    {
+      mobius::py::set_runtime_error (e.what ());
+    }
+
+  // Return value
+  return ret;
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief <i>set_tag</i> method implementation
+//! \param self Object
+//! \param args Argument list
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+static PyObject *
+tp_f_set_tag (model_evidence_o *self, PyObject *args)
+{
+  // Parse input args
+  std::string arg_name;
+
+  try
+    {
+      arg_name = mobius::py::get_arg_as_std_string (args, 0);
+    }
+  catch (const std::exception& e)
+    {
+      mobius::py::set_invalid_type_error (e.what ());
+      return nullptr;
+    }
+
+  // Execute C++ function
+  try
+    {
+      self->obj->set_tag (arg_name);
+    }
+  catch (const std::exception& e)
+    {
+      mobius::py::set_runtime_error (e.what ());
+      return nullptr;
+    }
+
+  // return None
+  return mobius::py::pynone ();
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief <i>reset_tag</i> method implementation
+//! \param self Object
+//! \param args Argument list
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+static PyObject *
+tp_f_reset_tag (model_evidence_o *self, PyObject *args)
+{
+  // Parse input args
+  std::string arg_name;
+
+  try
+    {
+      arg_name = mobius::py::get_arg_as_std_string (args, 0);
+    }
+  catch (const std::exception& e)
+    {
+      mobius::py::set_invalid_type_error (e.what ());
+      return nullptr;
+    }
+
+  // Execute C++ function
+  try
+    {
+      self->obj->reset_tag (arg_name);
+    }
+  catch (const std::exception& e)
+    {
+      mobius::py::set_runtime_error (e.what ());
+      return nullptr;
+    }
+
+  // return None
+  return mobius::py::pynone ();
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief <i>get_tags</i> method implementation
+//! \param self Object
+//! \param args Argument list
+//! \return Set of evidence tags
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+static PyObject *
+tp_f_get_tags (model_evidence_o *self, PyObject *)
+{
+  // Execute C++ function
+  PyObject *ret = nullptr;
+
+  try
+    {
+      ret = mobius::py::pyset_from_cpp_container (
+              self->obj->get_tags (),
+              mobius::py::pystring_from_std_string
+      );
+    }
+  catch (const std::exception& e)
+    {
+      mobius::py::set_runtime_error (e.what ());
+    }
+
+  // Return value
+  return ret;
+}
+
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 //! \brief Methods structure
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -370,6 +508,30 @@ static PyMethodDef tp_methods[] =
     METH_VARARGS,
     "Get attributes as dict"
   },
+  {
+    (char *) "has_tag",
+    (PyCFunction) tp_f_has_tag,
+    METH_VARARGS,
+    "Check if evidence has a given tag"
+  },
+  {
+    (char *) "set_tag",
+    (PyCFunction) tp_f_set_tag,
+    METH_VARARGS,
+    "Set tag"
+  },
+  {
+    (char *) "reset_tag",
+    (PyCFunction) tp_f_reset_tag,
+    METH_VARARGS,
+    "Reset tag"
+  },
+  {
+    (char *) "get_tags",
+    (PyCFunction) tp_f_get_tags,
+    METH_VARARGS,
+    "Get tags"
+  },
   {NULL, NULL, 0, NULL} // sentinel
 };
 
